@@ -1,7 +1,6 @@
-#!/usr/bin/python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2013 Gauvain Pocentek <gauvain@pocentek.net>
+# Copyright (C) 2013-2014 Gauvain Pocentek <gauvain@pocentek.net>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -21,11 +20,11 @@ import requests
 import sys
 
 __title__ = 'python-gitlab'
-__version__ = '0.6'
+__version__ = '0.7'
 __author__ = 'Gauvain Pocentek'
 __email__ = 'gauvain@pocentek.net'
 __license__ = 'LGPL3'
-__copyright__ = 'Copyright 2013 Gauvain Pocentek'
+__copyright__ = 'Copyright 2013-2014 Gauvain Pocentek'
 
 
 class jsonEncoder(json.JSONEncoder):
@@ -89,6 +88,8 @@ class Gitlab(object):
         self.email = email
         self.password = password
         self.ssl_verify = ssl_verify
+        # Gitlab should handle UTF-8
+        self.gitlab_encoding = 'UTF-8'
 
     def auth(self):
         """Performs an authentication using either the private token, or the
@@ -186,11 +187,12 @@ class Gitlab(object):
             raise GitlabListError('Missing attribute(s): %s' %
                                   ", ".join(missing))
 
-        url = obj_class._url % kwargs
+        args = _sanitize_dict(kwargs)
+        url = obj_class._url % args
         url = '%s%s' % (self._url, url)
-        if kwargs:
+        if args:
             url += "?%s" % ("&".join(
-                   ["%s=%s" % (k, v) for k, v in kwargs.items()]))
+                   ["%s=%s" % (k, v) for k, v in args.items()]))
 
         try:
             r = requests.get(url, headers=self.headers, verify=self.ssl_verify)
@@ -224,7 +226,7 @@ class Gitlab(object):
             raise GitlabListError('Missing attribute(s): %s' %
                                   ", ".join(missing))
 
-        url = obj_class._url % kwargs
+        url = obj_class._url % _sanitize_dict(kwargs)
         if id is not None:
             url = '%s%s/%s' % (self._url, url, str(id))
         else:
@@ -246,8 +248,9 @@ class Gitlab(object):
             raise GitlabGetError('%d: %s' % (r.status_code, r.text))
 
     def delete(self, obj):
-        url = obj._url % obj.__dict__
-        url = '%s%s/%s' % (self._url, url, str(obj.id))
+        args = _sanitize_dict(obj.__dict__)
+        url = obj._url % args
+        url = '%s%s/%s' % (self._url, url, args['id'])
 
         try:
             r = requests.delete(url,
@@ -274,9 +277,14 @@ class Gitlab(object):
             raise GitlabCreateError('Missing attribute(s): %s' %
                                     ", ".join(missing))
 
-        url = obj._url % obj.__dict__
+        args = _sanitize_dict(obj.__dict__)
+        url = obj._url % args
         url = '%s%s' % (self._url, url)
 
+        for k, v in obj.__dict__.items():
+            if type(v) == bool:
+                obj.__dict__[k] = 1 if v else 0
+
         try:
             r = requests.post(url, obj.__dict__,
                               headers=self.headers,
@@ -293,16 +301,19 @@ class Gitlab(object):
             raise GitlabCreateError('%d: %s' % (r.status_code, r.text))
 
     def update(self, obj):
-        url = obj._url % obj.__dict__
+        args = _sanitize_dict(obj.__dict__)
+        url = obj._url % args
         url = '%s%s/%s' % (self._url, url, str(obj.id))
 
         # build a dict of data that can really be sent to server
         d = {}
         for k, v in obj.__dict__.items():
-            if type(v) in (int, str, bool):
+            if type(v) in (int, str):
                 d[k] = str(v)
+            elif type(v) == bool:
+                d[k] = 1 if v else 0
             elif type(v) == unicode:
-                d[k] = str(v.encode(sys.stdout.encoding, "replace"))
+                d[k] = str(v.encode(self.gitlab_encoding, "replace"))
 
         try:
             r = requests.put(url, d,
@@ -446,6 +457,20 @@ class Gitlab(object):
         return self._getListOrObject(Team, id, **kwargs)
 
 
+def _get_display_encoding():
+    return sys.stdout.encoding or sys.getdefaultencoding()
+
+
+def _sanitize(value):
+    if type(value) in (str, unicode):
+        return value.replace('/', '%2F')
+    return value
+
+
+def _sanitize_dict(src):
+    return {k: _sanitize(v) for k, v in src.items()}
+
+
 class GitlabObject(object):
     _url = None
     _returnClass = None
@@ -574,7 +599,7 @@ class GitlabObject(object):
             s = ", ".join([GitlabObject._obj_to_str(x) for x in obj])
             return "[ %s ]" % s
         elif isinstance(obj, unicode):
-            return obj.encode(sys.stdout.encoding, "replace")
+            return obj.encode(_get_display_encoding(), "replace")
         else:
             return str(obj)
 
@@ -585,8 +610,8 @@ class GitlabObject(object):
             if k == self.idAttr:
                 continue
             v = self.__dict__[k]
-            pretty_k = k.replace('_', '-').encode(sys.stdout.encoding,
-                                                  "replace")
+            pretty_k = k.replace('_', '-')
+            pretty_k = pretty_k.encode(_get_display_encoding(), "replace")
             if isinstance(v, GitlabObject):
                 if depth == 0:
                     print("%s:" % pretty_k)
@@ -606,9 +631,9 @@ class GitlabObject(object):
 class UserKey(GitlabObject):
     _url = '/users/%(user_id)s/keys'
     canGet = False
-    canList = False
+    canList = True
     canUpdate = False
-    canDelete = False
+    canDelete = True
     requiredCreateAttrs = ['user_id', 'title', 'key']
 
 
@@ -702,11 +727,11 @@ class Issue(GitlabObject):
 class ProjectBranch(GitlabObject):
     _url = '/projects/%(project_id)s/repository/branches'
     idAttr = 'name'
-    canDelete = False
     canUpdate = False
-    canCreate = False
     requiredGetAttrs = ['project_id']
     requiredListAttrs = ['project_id']
+    requiredCreateAttrs = ['project_id', 'branch_name', 'ref']
+    requiredDeleteAttrs = ['project_id']
     _constructorTypes = {'commit': 'ProjectCommit'}
 
     def protect(self, protect=True):
@@ -736,8 +761,8 @@ class ProjectCommit(GitlabObject):
     shortPrintAttr = 'title'
 
     def diff(self):
-        url = '/projects/%(project_id)s/repository/commits/%(commit_id)s/diff' % \
-              {'project_id': self.project_id, 'commit_id': self.id}
+        url = ('/projects/%(project_id)s/repository/commits/%(commit_id)s/diff'
+               % {'project_id': self.project_id, 'commit_id': self.id})
         r = self.gitlab.rawGet(url)
         if r.status_code == 200:
             return r.json()
@@ -872,7 +897,7 @@ class ProjectMilestone(GitlabObject):
     requiredListAttrs = ['project_id']
     requiredGetAttrs = ['project_id']
     requiredCreateAttrs = ['project_id', 'title']
-    optionalCreateAttrs = ['description', 'due_date']
+    optionalCreateAttrs = ['description', 'due_date', 'state_event']
     shortPrintAttr = 'title'
 
 
@@ -953,11 +978,6 @@ class Project(GitlabObject):
                                      project_id=self.id,
                                      **kwargs)
 
-    def File(self, id=None, **kwargs):
-        return self._getListOrObject(ProjectFile, id,
-                                     project_id=self.id,
-                                     **kwargs)
-
     def Hook(self, id=None, **kwargs):
         return self._getListOrObject(ProjectHook, id,
                                      project_id=self.id,
